@@ -1,4 +1,4 @@
-__version__ = '0.14.2'
+__version__ = '0.15.0'
 
 def can_process(input_file):
     '''
@@ -7,7 +7,7 @@ def can_process(input_file):
     Args:
         input_file: Path to a file.
     '''
-    if input_file.lower().endswith(('.docx', '.kxliff', '.odp', '.ods', '.odt', '.po', '.sdlxliff', '.txt', '.xliff')):
+    if input_file.lower().endswith(('.docx', '.json', '.kxliff', '.odp', '.ods', '.odt', '.po', '.sdlxliff', '.txt', '.xliff')):
         return True
     else:
         return False
